bl_info = {
    "name": "Context Switcher",
    "blender": (4, 2, 1),
    "category": "User Interface",
}

import bpy
import rna_keymap_ui
from bpy.types import Operator, AddonPreferences
import logging
import datetime
import os

# Set up logging to file
log_file = os.path.join(os.path.expanduser("~"), "context_switcher_debug.log")
logging.basicConfig(
    filename=log_file,
    level=logging.DEBUG,
    format='%(asctime)s - %(levelname)s - %(message)s'
)
logger = logging.getLogger(__name__)

def log_keymap_state(prefix=""):
    """Helper function to log current keymap state"""
    wm = bpy.context.window_manager
    logger.debug(f"\n{prefix} Keymap State Check:")
    
    # Check user keyconfig
    kc_user = wm.keyconfigs.user
    if kc_user:
        logger.debug("User Keyconfig exists")
        km = kc_user.keymaps.get('Window')
        if km:
            logger.debug("Window keymap exists in user config")
            for kmi in km.keymap_items:
                if kmi.idname == SCREEN_OT_context_switcher.bl_idname:
                    logger.debug(f"Found keymap item: type={kmi.type}, value={kmi.value}, map_type={kmi.map_type}")
        else:
            logger.debug("No Window keymap in user config")
    else:
        logger.debug("No user keyconfig")

    # Check addon keyconfig
    kc_addon = wm.keyconfigs.addon
    if kc_addon:
        logger.debug("Addon Keyconfig exists")
        km = kc_addon.keymaps.get('Window')
        if km:
            logger.debug("Window keymap exists in addon config")
            for kmi in km.keymap_items:
                if kmi.idname == SCREEN_OT_context_switcher.bl_idname:
                    logger.debug(f"Found keymap item: type={kmi.type}, value={kmi.value}, map_type={kmi.map_type}")
        else:
            logger.debug("No Window keymap in addon config")
    else:
        logger.debug("No addon keyconfig")

class SCREEN_OT_context_switcher(Operator):
    bl_idname = "screen.context_switcher"
    bl_label = "Context Switcher"
    bl_description = "Open the context switcher at the mouse cursor location"
    bl_options = {'REGISTER', 'UNDO'}

    def switch_workspace(self, context, workspace_name):
        for ws in bpy.data.workspaces:
            if ws.name == workspace_name:
                context.window.workspace = ws
                return {'FINISHED'}
        return {'CANCELLED'}

    def draw(self, context):
        layout = self.layout
        
        area_types = {
            "General": [
                ('VIEW_3D', "3D Viewport", 'VIEW3D'),
                ('IMAGE_EDITOR', "Image Editor", 'IMAGE'),
                ('UV', "UV Editor", 'UV'),
                ('CompositorNodeTree', "Compositor", 'NODE_COMPOSITING'),
                ('TextureNodeTree', "Texture Node Editor", 'NODE_TEXTURE'),
                ('GeometryNodeTree', "Geometry Node Editor", 'NODETREE'),
                ('ShaderNodeTree', "Shader Editor", 'NODE_MATERIAL'),
                ('SEQUENCE_EDITOR', "Video Sequencer", 'SEQUENCE'),
                ('CLIP_EDITOR', "Movie Clip Editor", 'TRACKER'),
            ],
            "Animation": [
                ('DOPESHEET', "Dope Sheet", 'ACTION'),
                ('TIMELINE', "Timeline", 'TIME'),
                ('FCURVES', "Graph Editor", 'GRAPH'),
                ('DRIVERS', "Drivers", 'DRIVER'),
                ('NLA_EDITOR', "Nonlinear Animation", 'NLA'),
            ],
            "Scripting": [
                ('TEXT_EDITOR', "Text Editor", 'TEXT'),
                ('CONSOLE', "Python Console", 'CONSOLE'),
                ('INFO', "Info", 'INFO'),
            ],
            "Data": [
                ('OUTLINER', "Outliner", 'OUTLINER'),
                ('PROPERTIES', "Properties", 'PROPERTIES'),
                ('FILES', "File Browser", 'FILEBROWSER'),
                ('ASSETS', "Asset Browser", 'ASSET_MANAGER'),
                ('SPREADSHEET', "Spreadsheet", 'SPREADSHEET'),
                ('PREFERENCES', "Preferences", 'PREFERENCES'),
            ]
        }

        row = layout.row()
        
        # Add workspaces column first
        col = row.column()
        col.label(text="Workspaces")
        
        # Get current workspace and area type
        current_workspace = context.workspace.name
        current_area_type = context.area.ui_type
        
        # List all workspaces
        for ws in bpy.data.workspaces:
            op = col.operator("screen.context_switcher_workspace", 
                            text=ws.name, 
                            icon='WORKSPACE',
                            depress=(ws.name == current_workspace))
            op.workspace_name = ws.name
        
        row.separator()
        
        for group, types in area_types.items():
            col = row.column()
            col.label(text=group)
            for identifier, name, icon in types:
                props = col.operator("wm.context_set_enum", 
                                   text=name, 
                                   icon=icon,
                                   depress=(identifier == current_area_type))
                props.data_path = "area.ui_type"
                props.value = identifier

    def invoke(self, context, event):
        return context.window_manager.invoke_popup(self, width=700)

    def execute(self, context):
        return {'FINISHED'}

class SCREEN_OT_context_switcher_workspace(Operator):
    bl_idname = "screen.context_switcher_workspace"
    bl_label = "Switch Workspace"
    bl_description = "Switch to the selected workspace"
    bl_options = {'INTERNAL'}

    workspace_name: bpy.props.StringProperty()

    def execute(self, context):
        for ws in bpy.data.workspaces:
            if ws.name == self.workspace_name:
                context.window.workspace = ws
                break
        return {'FINISHED'}

class AddonPreferences(bpy.types.AddonPreferences):
    bl_idname = __name__

    def draw(self, context):
        layout = self.layout
        box = layout.box()
        col = box.column()
        col.label(text="Keymap List:", icon="KEYINGSET")

        wm = context.window_manager
        kc = wm.keyconfigs.user
        if not kc:
            col.label(text="No user keyconfig available", icon='ERROR')
            return

        km = kc.keymaps.get('Window')
        if km:
            for kmi in km.keymap_items:
                if kmi.idname == SCREEN_OT_context_switcher.bl_idname:
                    col.context_pointer_set("keymap", km)
                    rna_keymap_ui.draw_kmi([], kc, km, kmi, col, 0)
                    return

        # No keymap found, show add button
        row = col.row()
        row.operator(SCREEN_OT_context_switcher_add_keymap.bl_idname, 
                    text="Add Keyboard Shortcut", 
                    icon='ADD')

class SCREEN_OT_context_switcher_add_keymap(Operator):
    bl_idname = "screen.context_switcher_add_keymap"
    bl_label = "Add Context Switcher Keymap"
    bl_description = "Add keyboard shortcut for Context Switcher"
    bl_options = {'REGISTER', 'INTERNAL'}

    def execute(self, context):
        wm = context.window_manager
        kc = wm.keyconfigs.user
        if kc:
            km = kc.keymaps.get('Window')
            if not km:
                km = kc.keymaps.new(name='Window', space_type='EMPTY')
            kmi = km.keymap_items.new(
                SCREEN_OT_context_switcher.bl_idname,
                type='NONE',
                value='PRESS'
            )
        return {'FINISHED'}

def draw_menu(self, context):
    self.layout.operator(SCREEN_OT_context_switcher.bl_idname)

def register():
    bpy.utils.register_class(SCREEN_OT_context_switcher_workspace)
    bpy.utils.register_class(SCREEN_OT_context_switcher)
    bpy.utils.register_class(SCREEN_OT_context_switcher_add_keymap)
    bpy.utils.register_class(AddonPreferences)
    bpy.types.VIEW3D_MT_view.append(draw_menu)

def unregister():
    bpy.types.VIEW3D_MT_view.remove(draw_menu)
    bpy.utils.unregister_class(SCREEN_OT_context_switcher_workspace)
    bpy.utils.unregister_class(SCREEN_OT_context_switcher)
    bpy.utils.unregister_class(SCREEN_OT_context_switcher_add_keymap)
    bpy.utils.unregister_class(AddonPreferences)

if __name__ == "__main__":
    register()